#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>
#include <tapidbus/tapidbus.h>

static DBusMessage *handle_test_method (TapiDBus *td, DBusMessage *msg, void *user_data);
static void         handle_test_signal (TapiDBus *td, DBusMessage *msg, void *user_data);
static DBusMessage *handle_test_method1 (TapiDBus *td, DBusMessage *msg, void *user_data);
static void         handle_test_signal1 (TapiDBus *td, DBusMessage *msg, void *user_data);

static GMainLoop *loop;

static TapiDBusMethod methods[] = {
    { "org.maemo.TapiDBus", "TestMethod", "s", handle_test_method },
    { NULL }
};

static TapiDBusSignal signals[] = {
    { "org.maemo.TapiDBus", "TestSignal", "i", handle_test_signal },
    { NULL }
};

static TapiDBusMethod methods1[] = {
    { "org.maemo.TapiDBus", "TestMethod", "s", handle_test_method1 },
    { NULL }
};

static TapiDBusSignal signals1[] = {
    { "org.maemo.TapiDBus", "TestSignal", "i", handle_test_signal1 },
    { NULL }
};

int main()
{
    TapiDBus *td, *td1;

    loop = g_main_loop_new (NULL, FALSE);
    td = tapidbus_new_with_service ("org.maemo.TapiDBus", "/org/maemo/TapiDBus/c1", methods, signals);
    td1 = tapidbus_new_with_service ("org.maemo.TapiDBus", "/org/maemo/TapiDBus/c2", methods1, signals1);
    g_main_loop_run (loop);
    tapidbus_destroy (td);
    tapidbus_destroy (td1);
    return 0;
}

static DBusMessage * 
handle_test_method (TapiDBus    *td,
                    DBusMessage *message, 
                    void        *user_data) 
{
    DBusError error;
    gchar *str;

    dbus_error_init (&error);
    if (!dbus_message_get_args (message, &error,
                DBUS_TYPE_STRING, &str,
                DBUS_TYPE_INVALID)) {
        dbus_error_free (&error);
        return NULL;
    }
    
    g_print ("%s (%d): message received: %s\n", 
            __FUNCTION__, __LINE__, str);
    return tapidbus_build_method_return (message, 
            DBUS_TYPE_STRING,
            __PRETTY_FUNCTION__,
            DBUS_TYPE_INVALID);
}

static void 
handle_test_signal (TapiDBus    *td,
                    DBusMessage *message, 
                    void        *user_data) 
{
    DBusError error;
    gint i;

    dbus_error_init (&error);
    if (!dbus_message_get_args (message, &error,
                DBUS_TYPE_INT32, &i,
                DBUS_TYPE_INVALID)) {
        dbus_error_free (&error);
        return;
    }
    
    g_print ("%s (%d): message received: %i\n", 
            __FUNCTION__, __LINE__, i);
}

static DBusMessage * 
handle_test_method1 (TapiDBus    *td,
                     DBusMessage *message, 
                     void        *user_data) 
{
    DBusError error;
    gchar *str;

    dbus_error_init (&error);
    if (!dbus_message_get_args (message, &error,
                DBUS_TYPE_STRING, &str,
                DBUS_TYPE_INVALID)) {
        dbus_error_free (&error);
        return NULL;
    }
    
    g_print ("%s (%d): message received 1: %s\n", 
            __FUNCTION__, __LINE__, str);
    return tapidbus_build_method_return (message, 
            DBUS_TYPE_STRING,
            __PRETTY_FUNCTION__,
            DBUS_TYPE_INVALID);
}

static void 
handle_test_signal1 (TapiDBus    *td,
                     DBusMessage *message, 
                     void        *user_data) 
{
    DBusError error;
    gint i;

    dbus_error_init (&error);
    if (!dbus_message_get_args (message, &error,
                DBUS_TYPE_INT32, &i,
                DBUS_TYPE_INVALID)) {
        dbus_error_free (&error);
        return;
    }
    
    g_print ("%s (%d): message received 1: %i\n", 
            __FUNCTION__, __LINE__, i);

    g_main_loop_quit (loop);
}
